//
// C++ Implementation: KernelTransformation
//
// Description: 
//
//
// Author: Marek Grochowski <grochu@fizyka.umk.pl>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "KernelTransformation.h"

Data * KernelTransformation::Learn( Data * trainData )
{
    number_of_features = trainData->GetNumberOfFeatures();
    number_of_vectors = trainData->GetNumberOfVectors();
    
    int vectorsCount = 0;
    int *vectorsID = new int[number_of_vectors];
    
    RandomID rand(number_of_vectors);
    
  //  vectorsID[vectorsCount] = rand.Next();
    vectorsID[vectorsCount] = 0;
    vectorsCount++;
    
    for(int i=1;i<number_of_vectors;i++)
    {
            bool add = true;
           // int vecID = rand.Next();
             int vecID = i;
             //uwaga - wypadaloby zmniejszyc liczbe operacji poprzez zapamietanie 
             // dlugosci wektorow zamiast obliczac je za kazdym razem od nowa wewnatrz funkcji Cryterium
            for(int k=0;k<vectorsCount && add ;k++)
                if(!Cryterium(trainData->GetInput( vecID),trainData->GetInput( vectorsID[k]),number_of_features))
                {
                    add = false;
                    break;
                }
             
            if(add)
            {
                vectorsID[vectorsCount] = vecID;
                vectorsCount++;
            }
    }
    
    vectors = new float * [vectorsCount];
    if(!vectors) throw new MemoryException("No memory for new vectors");

    for(long i=0;i<vectorsCount;i++)
    {
        vectors[i] = new float[number_of_features];
        if(!vectors[i]) throw new MemoryException("No memory for new vectors");
        
        float *src = trainData->GetInput( vectorsID[i] );
        float *dst = vectors[i];
        
        for(long f=0;f<number_of_features;f++)
        {
            *dst = *src;
            dst++; src++;
        }
    }
    
    cout << endl;
    for(int k=0;k<vectorsCount;k++)
        cout <<  vectorsID[k] << " ";
    cout << endl;
    delete [] vectorsID;
    number_of_vectors = vectorsCount;
    //this->DisplayKernels();
        
    return Run(trainData);        
}

Data * KernelTransformation::Run( Data * dataToTransform )
{
    Data *newdata = new Data();
    if(!newdata) throw new MemoryException("No memory for new dataset in KernelTransformation");
    
    long data_f = dataToTransform->GetNumberOfFeatures();
    long data_v = dataToTransform->GetNumberOfVectors();
    long newdata_f = number_of_features + number_of_vectors;
    
    newdata->CreateEmptyDataTable( data_v , newdata_f );

    for(long i=0;i<data_v;i++)
    {
        float *src=dataToTransform->GetInput( i);
        float *dst=newdata->GetInput( i);
        
        for(long f=0;f<data_f;f++) dst[f]=src[f];   
        newdata->SetLabel( i, dataToTransform->GetLabel( i));
        
        for(int f=0;f<number_of_vectors;f++)
            dst[data_f+f]= KernelFunction(src,vectors[f],number_of_features);   
    }
    newdata->RefreshDataInfo();
    return newdata;
}

void KernelTransformation::Configure( Parameters & p )
{
}

KernelTransformation::~ KernelTransformation( )
{
    for(long i=0;i<number_of_vectors;i++)
        delete [] vectors[i];
    
    delete [] vectors;
}

void KernelTransformation::SetKernel( int i )
{
    if(i==1) KernelFunction = KernelFunctions::Gauss;
    else KernelFunction = KernelFunctions::InerProduct;

}

int KernelTransformation::Cryterium( float * v1, float * v2, int dim )
{
    float d1 = KernelFunctions::InerProduct( v1, v1,dim);
    float d2 = KernelFunctions::InerProduct( v2, v2,dim);
    float alfa = KernelFunctions::InerProduct( v1, v2,dim);
    float d = sqrt(fabs(d1*d2));
    if(d > 0.0 ) 
    {
        alfa = alfa/d;
    }else alfa = 1.0;
    
    if(alfa < a) return 1;
    else return 0;
}

void KernelTransformation::DisplayKernels( )
{
    for(int i=0;i<number_of_vectors;i++)
    {
        cout << "[ " << i << " ] ";
        for(int f=0;f < number_of_features;f++)
            cout << (*(vectors+i))[f] << " ";
        cout << endl;
    }

}
