//
// C++ Interface: LabelsReprezentation
//
// Description: 
//
//
// Author: Marek Grochowski <grochu@fizyka.umk.pl>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef _LABELS_REPREZENTATION_H
#define _LABELS_REPREZENTATION_H
#include <set>
#include <vector>
#include "exceptions.h"

typedef float Label;

using namespace std;

struct LabelInfo
{
    Label label;
    int numberOfVectors;
    float *reprezentationTable;
    
    LabelInfo():label(0.0),reprezentationTable(NULL),numberOfVectors(0)
            {   };
    LabelInfo(Label _label):label(_label),reprezentationTable(NULL),numberOfVectors(0)
    {
        };
    ~LabelInfo()
    {
        delete [] reprezentationTable;
    }
    void CreateDiagonalReprezentation(int id,int size)
    {
        if (reprezentationTable) delete [] reprezentationTable;
        reprezentationTable = new float[size];
        if(!reprezentationTable) throw new MemoryException("Diagonal reprezentation not created");
        for(int i=0;i<size;i++)
        {
         //   reprezentationTable[i] = (i==id) ? 1.0 : 2.0;
            if(i==id) reprezentationTable[i]=1;
            else reprezentationTable[i]=0;
         }
    }
};

class LabelsReprezentation
{
private:
    int numberOfLabels;
    vector<LabelInfo> labelsInfoTable;
    int Find(Label label);
    
public:
    void Clear(); 
    LabelsReprezentation();
    LabelsReprezentation(Label *labelTable,int size);
    void CreateDiagonalReprezentation(Label *labelTable,int size);
    float *Label2Output(Label label);
    Label GetLabel(int id);
    int GetNumberOfVectorsInClass(int c);
    int GetNumberOfLabels();
    ~LabelsReprezentation();
    void SetNumberOfVectors(int cl,int count)
    {
        labelsInfoTable[cl].numberOfVectors = count;
    }
    void IncreaseNumberOfVectors(Label l)
    {
        labelsInfoTable[Find(l)].numberOfVectors++;
    }
    
    friend ostream& operator<<(ostream &o,LabelsReprezentation &lr)
    {
        o << "Number of labels      " << lr.numberOfLabels;
        o << endl;
        for(int i=0;i<lr.numberOfLabels;i++)
        {
            Label l = lr.GetLabel( i );
            o << "Label : " << l;
            o << "\tcount=" << lr.GetNumberOfVectorsInClass(i);
            o << "\trepr= ";
            float *rep = lr.Label2Output( l );
            for(int k=0;k<lr.numberOfLabels;k++) o << rep[k] << " ";
            o << endl;
        }
        return o; 
    }
};
#endif
