#include "Tester.h"

/*
float CrossValidation::Mean()
{
	int i=(int)resultsTable.size();

	if(i!=0) mean_acc=sum/(float)i;
	return mean_acc;
}
*/
/*
float CrossValidation::Std()
{
	list<Results>::iterator it;
	
	float cum=0.0;
	int n=(int)resultsTable.size();
	if(n < 2) return 0;

	for(it=resultsTable.begin();it!=resultsTable.end();it++)
	{
		float diff=mean_acc-(*it).mean_acc;
		cum+=diff*diff;
	}
	
	std=sqrt(cum/(float)(n-1));

	return std;
}
*/

void SimpleTest::Run(Data *d)
{
	if(!model)	throw new Exception("Nie zdefiniowany model do testu");

	unsigned long nr_vec=d->GetNumberOfVectors();
	unsigned long correct=0;
	
	if(train_data)	
	{
		((Classifier*)model)->Reset();
		((Classifier*)model)->Learn(train_data);
		//blad treningowy
		SimpleTest st;
		st.AddModel(model);
		st.Run(train_data);
		train_results.AddResult(st.GetTestResults().mean_acc);
		train_results.Refresh();
	}

	for(unsigned long i=0;i<nr_vec;i++)
		if(((Classifier *)model)->Test(d->GetInput(i))==d->GetLabel(i)) correct++;

/*    for(unsigned long i=0;i<nr_vec;i++)
        cout << ((Classifier *)model)->Test(d->GetInput(i)) << " <-> " << d->GetLabel(i) << endl;*/
	
	test_results.AddResult((float)correct/(float)nr_vec);
	test_results.Refresh();
}

void SimpleTest::DisplayResults()
{
	if(train_data) cout << "Simple test: train accuracy = " << train_results.mean_acc * 100 << endl;
	cout << "Simple test: test accuracy = " << test_results.mean_acc * 100 << endl;
}

Results *SimpleTest::GetResults()
{
	return &test_results;
}


//------------------------------------ CV -----------------------------------------------
void CrossValidation::DisplayResults()
{
	list<float>::iterator it = test_results.accuracyTable.begin();
	list<float>::iterator itt = train_results.accuracyTable.begin();
	int i=0;
	cout << "Crossvalidation results (accuracy):" << endl;
	cout << " fold \t test \t train \n------------------------------\n";
	cout.precision(3);
	cout.setf(std::ios_base::fixed);
	while(it!=test_results.accuracyTable.end())
	{
		cout << "  " << i+1 << "\t" << *it << "\t" << *itt <<endl;
		it++;
		itt++;
		i++;
	}
	cout << "------------------------------\n" << endl;
	cout << "Mean= " << test_results.mean_acc << "\t" <<  train_results.mean_acc << endl;
	cout << " Std= " <<  test_results.std     << "\t" <<  train_results.std << endl;
}

Results *CrossValidation::GetResults()
{
	return &test_results;
}

void CrossValidation::Run(Data *d)
{
	if (!model) throw new Exception("Nie zdefiniowany model");

	if (nr_of_folds > d->GetNumberOfVectors()) nr_of_folds = d->GetNumberOfVectors();

	if(type==monte_carlo) MonteCarlo(d);
	if(type==stratified) Stratified(d);

	/* zbieranie wynikow */	
	SimpleTest st;
	st.AddModel(model);

	for(int i=0;i<nr_of_folds;i++)
	{	
		cout << "CrossValidation fold " << i+1 << endl;
		
		st.Reset();
		st.AddTrainData(folds[i].train);
		st.Run(folds[i].test);
		
		train_results.AddResult(st.GetTrainResults().mean_acc);
		test_results.AddResult(st.GetTestResults().mean_acc);
	}
	
	train_results.Refresh();
	test_results.Refresh();
}

void CrossValidation::Configure(Parameters &p)
{
	nr_of_folds = p.nr_cv;
	if(p.cv_method==0) type=stratified;
		else type=monte_carlo;
}

void CrossValidation::MonteCarlo(Data *d)
{
	Data dataCopy(*d);

	FoldsTable *f = new struct FoldsTable[nr_of_folds];
	if(!f) throw new MemoryException("crossvalidation monte carlo");

	unsigned long nr_vec=dataCopy.GetNumberOfVectors();
	long vectors_in_fold = (long)(nr_vec/nr_of_folds);
	long r = nr_vec%nr_of_folds;

	BitVector select(nr_vec);
   RandomID rand(nr_vec);
	
	for(int i=0;i<nr_of_folds;i++)
	{
		long vec_in_fold=vectors_in_fold;
		if(r) 
		{
			vec_in_fold++;
			r--;
		}
		
		select.Clear(0);
		
		while(vec_in_fold)
		{
			//select.SetBitOn(actual_vector);
			//actual_vector++;
			int id = rand.Next();
			select.SetBitOn(id);
			vec_in_fold--;
		}
		
		f[i].test=dataCopy.SelectVectors(select);
		f[i].train=dataCopy.SelectVectors(~select);
	}
	folds=f;
}

void CrossValidation::Stratified(Data *d)
{
	Data dataCopy(*d);

	FoldsTable *f = new struct FoldsTable[nr_of_folds];
	if(!f) throw new Exception("crossvalidation stratified");

	unsigned long nr_vec=dataCopy.GetNumberOfVectors();
	int nr_c = dataCopy.GetNumberOfClasses();

	long vectors_in_fold = (long)(nr_vec/nr_of_folds);
	long r = nr_vec%nr_of_folds;
	
	long *vec_in_fold_from_class = new long[nr_c];
	if(!vec_in_fold_from_class) throw new MemoryException("crossvalidation fold not created");

	long *class_rest = new long[nr_c];
	if(!class_rest) throw new Exception("crossvalidation stratified");

	for(int i=0;i<nr_c;i++) 
	{
		vec_in_fold_from_class[i]=(long)(dataCopy.GetNrVectorsInClass(i)/nr_of_folds);
		class_rest[i]=(long)(dataCopy.GetNrVectorsInClass(i)%nr_of_folds);
	}

	BitVector select(nr_vec);
	BitVector allvectors(nr_vec);
	allvectors.Clear(1);

   RandomID rand(nr_vec);

	for(int i=0;i<nr_of_folds;i++)
	{
		long vec_in_fold=vectors_in_fold;
		if(r) 
		{
			vec_in_fold++;
			r--;
		}
		select.Clear(0);
		
		long vec=0;
		
		for(int k=0;k<nr_c;k++)
		{
			long actual_class=vec_in_fold_from_class[k];
			
			while(actual_class)
			{
            vec = rand.Next();
            
				if(dataCopy.GetLabel(vec)==dataCopy.labelsInfo->GetLabel( k ) && allvectors[vec])
				{
					select.SetBitOn(vec);
					allvectors.SetBitOff(vec);
					actual_class--;
					vec_in_fold--;
				}
			}
 //        rand.Resample();
		}
		
		//reszta
		int k=0;
		while(vec_in_fold)			
		{
         vec=rand.Next();
			if(class_rest[k]  && allvectors[vec] && dataCopy.GetLabel(vec)==dataCopy.labelsInfo->GetLabel( k))
			{
					select.SetBitOn(vec);
					allvectors.SetBitOff(vec);
					vec_in_fold--;
					class_rest[k]--;
					k=(k+1)%nr_c;
			}
			if(!class_rest[k]) k=(k+1)%nr_c;
		}

		f[i].test=dataCopy.SelectVectors(select);
		f[i].train=dataCopy.SelectVectors(~select);
	}
	folds=f;
	
// 		for(int i=0;i<nr_of_folds;i++)
//     {
//     cout << "FOLD " << i << endl;
//     cout << "test " << i << endl;
//     f[i].test->DisplayData();
//     cout << "trening " << i << endl;
//     f[i].train->DisplayData();
//     
// 	
// 	}
// 	

	delete [] class_rest;
	delete [] vec_in_fold_from_class;
}

CrossValidation::~CrossValidation()
{
	if(folds)
	{
		for(int i=0;i<nr_of_folds;i++)
		{
			delete folds[i].test;
			delete folds[i].train;
		}
		delete [] folds;
	}
}


//---------------------------- XTEST ----------------------------------------------

void XTest::DisplayResults()
{
	list<float>::iterator it = test_results.accuracyTable.begin();
	list<float>::iterator itt = train_results.accuracyTable.begin();
	int i=0;
	cout << "\nX Test results (accuracy)" << endl;
	cout << " fold \t test \t train \n------------------------------\n";
	cout.precision(3);
	cout.setf(std::ios_base::fixed);
	while(it!=test_results.accuracyTable.end())
	{
		cout << "  " << i+1 << "\t" << *it << "\t" << *itt << endl;
		it++;
		itt++;
		i++;
	}
	cout << "------------------------------\n" << endl;
	cout << "Mean= " <<  test_results.mean_acc << "\t" <<  train_results.mean_acc << endl;
	cout << " Std= " <<  test_results.std << "\t" <<  train_results.std << endl;
}

Results *XTest::GetResults()
{
	return &test_results;
}

void XTest::Run(Data *d)
{
	if(!model) throw new Exception("XTest - nie zdefiniowano modelu");
	Tester *t=(Tester*)model;

	for(int i=0;i<nr_of_tests;i++)
	{
		t->Reset();
		t->Run(d);
		test_results.AddResult(t->GetTestResults().mean_acc);
		train_results.AddResult(t->GetTrainResults().mean_acc);
		cout <<  "Xtest fold " << i << endl;
		t->DisplayResults();
	}
	train_results.Refresh();
	test_results.Refresh();
}

void XTest::Configure(Parameters &p)
{
	nr_of_tests=p.nr_test;
	init_param=&p;
}
